<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2018 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2018.1/1742263
 */

use Users\Settings\ReviewPreferences;
use Users\Settings\TimePreferences;

return array(
    'users' => array(
        'maximum_dashboard_actions'  => 1000,
        'settings' => array(
            ReviewPreferences::REVIEW_PREFERENCES => array(
                ReviewPreferences::SHOW_COMMENTS_IN_FILES  => true,
                ReviewPreferences::VIEW_DIFFS_SIDE_BY_SIDE => true,
                ReviewPreferences::SHOW_SPACE_AND_NEW_LINE => false,
                ReviewPreferences::IGNORE_WHITESPACE       => false,
            ),
            TimePreferences::TIME_PREFERENCES => array(
                TimePreferences::DISPLAY  => 'Timeago', // Default to 'Timeago' but can be set to 'Timestamp'
            )
        )
    ),
    'avatars' => array(
        'http_url'  => 'http://www.gravatar.com/avatar/{hash}?s={size}&d={default}',
        'https_url' => 'https://secure.gravatar.com/avatar/{hash}?s={size}&d={default}'
    ),
    'security' => array(
        'login_exempt'  => array('login'),  // specify route id's which bypass require_login setting
        'mfa_routes'     => array(          // specify routes which are parth of the authentication workflow
            'login',
            'api/login/listmethods/listmethods',
            'api/login/initauth/initauth',
            'api/login/checkauth/checkauth',
            'api/login/checkauth/checkauthpoll',
            'logout'                        // added logout to routes as you can be P4 basic logged in but can't logout.

        ),
        'prevent_login' => array(),         // specify user ids which are not permitted to login to swarm
    ),
    'service_manager' => array(
        'factories' => array(
            'auth'      => function ($services) {
                // always use basic-auth credentials if they are specified
                // note: credentials, both basic and session, are not validated here, only retrieved
                $storage = new \Users\Authentication\Storage\BasicAuth($services->get('request'));
                $storage = $storage->read()
                    ? $storage
                    : new \Users\Authentication\Storage\Session(null, null, $services->get('session'));

                return new \Zend\Authentication\AuthenticationService($storage);
            },
            'user'      => function ($services) {
                $auth     = $services->get('auth');
                $p4Admin  = $services->get('p4_admin');
                $identity = (array) $auth->getIdentity() + array('id' => null);

                // if the user exists; return the full object
                if (Users\Model\User::exists($identity['id'], $p4Admin)) {
                    return Users\Model\User::fetchById($identity['id'], $p4Admin);
                }

                // user didn't exist; return an empty model (will have a null id)
                return new Users\Model\User($p4Admin);
            }
        ),
    ),
    'router' => array(
        'routes' => array(
            'home' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'index',
                    ),
                ),
            ),
            'login' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '/login[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'login',
                    ),
                ),
            ),
            'logout' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '/logout[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'logout',
                    ),
                ),
            ),
            'user' => array(
                'type' => 'Application\Router\Regex',
                'options' => array(
                    'regex'    => '/user(s?)/(?P<user>.+[^/])(/?)',
                    'spec'     => '/users/%user%',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'user'
                    ),
                ),
            ),
            'users' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '/users[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'users'
                    ),
                ),
            ),
            'follow' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '/follow/:type/:id[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'follow',
                        'type'       => null,
                        'id'         => null
                    ),
                ),
            ),
            'unfollow' => array(
                'type' => 'Zend\Mvc\Router\Http\Segment',
                'options' => array(
                    'route'    => '/unfollow/:type/:id[/]',
                    'defaults' => array(
                        'controller' => 'Users\Controller\Index',
                        'action'     => 'unfollow',
                        'type'       => null,
                        'id'         => null
                    ),
                ),
            ),
        ),
    ),
    'controllers' => array(
        'invokables' => array(
            'Users\Controller\Index' => 'Users\Controller\IndexController'
        ),
    ),
    'view_manager' => array(
        'template_map' => array(
            'users/index/index'  => __DIR__ . '/../view/users/index/index.phtml',
            'users/index/user'   => __DIR__ . '/../view/users/index/user.phtml',
        ),
        'template_path_stack'   => array(
            __DIR__ . '/../view',
        ),
    ),
    'view_helpers' => array(
        'invokables' => array(
            'user'                 => 'Users\View\Helper\User',
            'userLink'             => 'Users\View\Helper\UserLink',
            'avatar'               => 'Users\View\Helper\Avatar',
            'avatars'              => 'Users\View\Helper\Avatars',
            'notificationSettings' => 'Users\View\Helper\NotificationSettings',
            'userSettings'         => 'Users\View\Helper\Settings'
        ),
    ),
);
