/**
 * Perforce Swarm
 *
 * @copyright   2013-2018 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2018.1/1742263
 */

swarm.user = {
    userDetails: '',
    toolbar: '',
    csrf: '',


    settings: {
        init: function() {
            var $settings = $('#settingsForm');
            if ($settings.hasClass('readOnly')) {
                $settings.find(':checkbox').attr('disabled', true);
                $settings.find('.settingsSave').attr('disabled', true);
                $settings.find('#settingsReset').attr('disabled', true);
                $settings.find('#settingsCancel').attr('disabled', true);

                $settings.find('#time-format-display').attr('disabled', true);
            }
            swarm.user.settings.configureReset();
            // Set the token for the form.
            $('#settingsToken').attr('value', $('body').data('csrf'));
            swarm.user.settings.preventButtonDoubleClick();
        },
        configureReset: function() {
            $('#settingsReset').click(function () {
                $('#settings tbody input[type=checkbox]').each(function (i, item) {
                    item.checked = $(this).data('default');
                });
                // Set the selection boxes back to default settings.
                $('#settings tbody select').each(function (s, selection) {
                    var defaultSelected = this.dataset['default']; // Get default first
                    $(selection).find('option').each(function (x, option){ // Then find all the options for this select
                        // If the current option is the same name as the default set the selected to true
                        if (defaultSelected.toLowerCase() === option.id.toLowerCase()) {
                            option.selected = true;
                        }
                    });
                });
            });
        },
        preventButtonDoubleClick: function() {
            // Disable the save button once the user has clicked once.
            $('#settingsForm').submit(function (e) {
                $(this).find('button').prop('disabled', true);
            });
        }
    },
    login: function(message) {
        message = message && $('<div class="alert" />').text(message);

        if ($('.login-dialog.modal').length) {
            $('.login-inner').show();
            $('.login-dialog-inner').show();
            swarm.form.clearErrors('.login-dialog.modal form', true);
            $('.login-dialog.modal .login-dialog-inner  .modal-body').prepend(message);
            swarm.modal.show('.login-dialog.modal');
            return;
        }

        $.ajax({url: '/login', data: {format: 'partial'}}).done(function(data) {
            // for now we cannot use append due to performance issues with clearing password fields on long pages
            // @todo re-evaluate use of prepend after the following chrome bug is fixed http://crbug.com/180868
            $('body').prepend(data);
            $('.login-dialog.modal .login-dialog-inner .modal-body').prepend(message);
        });
    },

    enableMFA: function(form) {
        swarm.form.clearErrors(form);
        $.ajax({url: '/api/v9/login/listmethods', data: {format: 'json'}}).done(function(data) {
            var methods     = data.results.methods || data.results.method;
            var methodInput = $('#login2AuthMethod').empty();
            $.each(methods, function(index){
                var method = methods[index];
                var value  = method.methodName;
                var label  = method.methodDesc;
                methodInput.append("<label><input type='radio' name='method' value='" + value + "'/>" + label + "</label>");
            });
            $('.modal-body input[name="method"]').change(function (e) {
                $('.loginlist-dialog-inner #loginlistContinue').prop('disabled', false);
            });
            $("div[class$='-inner'],div[class*='-inner ']").find(".login").hide();
            $('.loginlist-dialog-inner').show().find('form').unbind();
            // need to disable the button after show
            $('.loginlist-dialog-inner #loginlistContinue').prop('disabled', true);
            $('.loginlist-dialog-inner').find('form').submit(function (e) {
                var form = this;
                swarm.form.clearErrors(form);
                e.preventDefault();
                swarm.form.post('/api/v9/login/initauth', '.loginlist-dialog form', function (response) {
                    if (200 === response.code) {
                        // Either new form or poll
                        if (response.option.hasOwnProperty("responseMethod") && "prompt" === response.option.responseMethod) {
                            // Success message is the
                            swarm.user.enableMFAPrompt(form, response.results);
                        } else {
                            // Poll for authenticated
                            swarm.user.enableMFAPoll(response.results);
                        }
                    }
                }, $('.loginlist-dialog .modal-body'));
            });
        });
    },
    enableMFAPrompt: function(form, results) {
        $('#mfaprompt #successMsg').text(results.successMsg);
        if (results.challenge) {
            $('#mfaprompt #challenge').text(results.challenge);
        }
        $("div[class$='-inner'],div[class*='-inner ']").find(".login").hide();
        $('.login2-dialog-inner').show().find('form').submit(function(e) {
            var form = this;
            swarm.form.clearErrors(form);
            e.preventDefault();
            swarm.form.post('/api/v9/login/checkauth', '.login2-dialog form', function(response) {
                if (200 === response.code) {
                    swarm.user.enableMFASuccess();
                }
            }, $('.login2-dialog .modal-body'));
        });
    },
    enableMFAPoll: function(results){
        $("div[class$='-inner'],div[class*='-inner ']").find(".login").hide();
        $('.login2-poll-inner').show();
        $('.pollMessage').text(results.successMsg);
        swarm.user.pollMFA();
    },
    pollMFA: function(){
        $.ajax({url: '/api/v9/login/checkauth', data: {format: 'json'}}).done(function(response) {
            if (200 === response.code) {
                if( response.mfaStatus.toString().includes("mfaPending")){
                    setTimeout(swarm.user.pollMFA, 5000);
                } else if(response.mfaStatus.toString().includes("mfaFailed")){
                    swarm.user.enableMFAFailed();
                } else {
                    swarm.user.enableMFASuccess();
                }
            }
        });
    },
    enableMFASuccess: function(){
        $("div[class$='-inner'],div[class*='-inner ']").find(".login").hide();
        $('.login2-success-inner').show();
        $(document).trigger($.extend($.Event('swarm-login'), {
            user:    swarm.user.userDetails,
            toolbar: swarm.user.toolbar,
            csrf:    swarm.user.csrf
        }));
        setTimeout(function(){
            // redirect the user to the homepage
            window.location = "/";
        },1500);
    },
    enableMFAFailed: function(){
        $("div[class$='-inner'],div[class*='-inner ']").find(".login").hide();
        $('.login2-failed-inner').show();
        setTimeout(function(){
            // redirect the user to the homepage
            swarm.user.enableMFA('');
        },1500);
    },
    loginDialog: function(e, partial, statusCode) {
        var form = this;
        e.preventDefault();
        swarm.form.post('/login', '.login-dialog form', function(response){
            if (response.isValid) {
                if (partial) {
                    if (response.user && response.user.mfa && response.user.mfa.indexOf('required') !== -1) {
                        swarm.user.enableMFA(form);
                        swarm.user.userDetails = response.user;
                        swarm.user.toolbar     = response.toolbar;
                        swarm.user.csrf        = response.csrf;
                    } else {
                        if ($('body').is('.login-reload')) {
                            return window.location.reload();
                        }
                        $(document).trigger($.extend($.Event('swarm-login'), {
                            user:    response.user,
                            toolbar: response.toolbar,
                            csrf:    response.csrf
                        }));
                        $('.login-dialog').modal('hide');
                    }
                } else if (statusCode === 401) {
                    // try reloading the active page after login if the
                    // initial page load had an 'unauthorized' statusCode
                    if (response.user && response.user.mfa && response.user.mfa.indexOf('required') !== -1) {
                        swarm.user.enableMFA(form);
                    } else {
                        window.location.reload();
                    }
                } else {
                    if (response.user && response.user.mfa && response.user.mfa.indexOf('required') !== -1) {
                        swarm.user.enableMFA(form);
                    } else {
                        // redirect the user to the homepage
                        window.location = "/";
                    }
                }
            }
        }, '.login-dialog .modal-body');
    },
    loginStatus: function() {
        $.ajax({url: '/api/v9/login/checkauth'}).done(function(data) {
            var errorMessage = '';
            if (data.error) {
                errorMessage = data.error;
            }
            if (errorMessage.indexOf("Second factor") > 0 || errorMessage.indexOf("init-auth") > 0
            || data.mfaStatus === 'mfaFailed'){
                swarm.user.login(swarm.t('Require Second Factor authentication'));
            } else if (errorMessage.indexOf("Unauthorized") > 0) {
                swarm.user.login(swarm.t('Your session has expired.'));
            }
        });
    },
    logoutUser: function (force) {
        if ($('body').hasClass('authenticated') || force === true) {
            $.ajax({url: '/logout'}).always(function() {
                location.reload();
            });
        }
    },

    follow: function(type, id, button) {
        button = $(button);
        swarm.form.disableButton(button);
        button.tooltip('destroy');

        // if we are already following, unfollow
        var unfollow = button.is('.following'),
            action   = unfollow ? 'unfollow' : 'follow';

        $.post(
            '/' + action + '/' + encodeURIComponent(type) + '/' + encodeURIComponent(id),
            function(response) {
                swarm.form.enableButton(button);
                if (response.isValid) {
                    // update button text (e.g. 'Follow' -> 'Unfollow' and vice-versa)
                    button.text(unfollow ? swarm.t('Follow') : swarm.t('Unfollow'));
                    button.toggleClass('following');

                    // indicate success via a temporary tooltip.
                    button.tooltip({
                        title:   unfollow ? swarm.t('No longer following %s', [id]) : swarm.t('Now following %s', [id]),
                        trigger: 'manual'
                    }).tooltip('show');

                    // update the UI immediately
                    swarm.user.updateFollowersSidebar(action);

                    setTimeout(function(){
                        button.tooltip('destroy');
                    }, 3000);
                }
            }
        );

        return false;
    },

    unfollowalldialog: function() {
        $('#unfollow-modal').modal('show');
    },

    unfollowall: function(user, button) {
        button = $(button);
        swarm.form.disableButton(button);
        button.tooltip('destroy');

        $.post(
            '/api/v8/users/' + user + '/unfollowall',
            function(response) {
                swarm.form.enableButton(button);
                if (response.isValid) {
                    location.reload();
                }
            }
        );
        return false;
    },

    updateFollowersSidebar: function(action) {
        var user      = swarm.user.getAuthenticatedUser(),
            counter   = $('.profile-sidebar .metrics .followers .count'),
            followers = $('.profile-sidebar .followers.profile-block'),
            avatars   = followers.find('.avatars');

        // nothing to do if not authenticated
        if (!user) {
            return;
        }

        // change the opacity of the avatar to indicate it's follow/unfollow state
        var avatar = avatars.find('img[data-user="' + user.id + '"]').closest('span');
        avatar.css('opacity', (action === 'follow' ? 1 : 0.2));

        // if we are following, but the avatar wasn't already in the page, we need to add it
        if (action === 'follow' && !avatar.length) {
            // may need to build a new row if one doesn't exist, or is full
            var row = avatars.find('> div').last();
            row     = (row.length && row.children().length < 5 && row) || $('<div />').appendTo(avatars);
            $('<span class="border-box" />').html(user.avatar).appendTo(row);
            followers.removeClass('hidden');
        }

        counter.text(parseInt(counter.text(), 10) + (action === 'follow' ? 1 : -1));
    },

    getAuthenticatedUser: function() {
        return $('body').data('user');
    }
};